
#if defined(GL_ES)
#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
precision highp int;
#else
precision mediump float;
precision mediump int;
#endif
#endif

varying vec4 Position;

#if defined(GL_ES)
varying vec2 v_texCoord;
#else
#define v_texCoord gl_TexCoord[0]
#define inTexCoord0 gl_MultiTexCoord0
#endif

uniform vec4 UserData1;
uniform vec4 UserData2;

#define AspectRatio UserData1.x
#define TransitionTime UserData1.y
#define RunningTime UserData1.z
#define Alpha UserData1.w

#define mDepth UserData2.x
#define HasLatestScreenGrab UserData2.y

uniform sampler2D Texture1Sampler;
uniform sampler2D Texture2Sampler;
uniform sampler2D Texture3Sampler;
uniform sampler2D Texture4Sampler;

void main() 
{	
	vec4 Color;
	vec4 AniTex;
	vec4 StatTex;
	vec2 texCoord = v_texCoord.xy;
	float speed = 10.0;
	float fire;
	float gradientMap;
	float fireMask;
	float distortion;
	float embers;
	float theTime;
	
	//get Static Texture
	StatTex = texture2D(Texture1Sampler, v_texCoord.xy);
	distortion = StatTex.g;
	gradientMap = StatTex.b;
	
	//animate Texture
	theTime = RunningTime * 0.04;
	theTime = mod(theTime, 1.0);

	texCoord.y += (theTime*speed*2.0); //scroll speed
	texCoord.x += ((distortion/10.0)+(theTime)); //distortion
	AniTex = texture2D(Texture1Sampler, texCoord.xy);
	fire = AniTex.a;
	embers = AniTex.r;
	
	//Modify gradientMap
	if (TransitionTime < 0.5)
	{
		gradientMap = ((1.0 + (-2.0*TransitionTime)) + gradientMap);
		embers = (embers - (1.0 + (-2.0*TransitionTime)));
	}
	if (TransitionTime > 0.5)
	{
		gradientMap = ((2.0*(TransitionTime - 0.5)) + gradientMap);
		embers = (embers - (2.0*(TransitionTime - 0.5)));
	}

	//create Mask
	fireMask = ((gradientMap - 0.5)*2.0) + fire;//addSub
	fireMask = 1.0 - fireMask; //invert mask
	fireMask = fireMask + embers;
	fireMask = clamp(fireMask,0.01,0.99);
	vec4 gradient = texture2D(Texture2Sampler, vec2(fireMask, 0.5));
	Color = vec4(gradient.r, gradient.g, gradient.b, fireMask);
    
	if (HasLatestScreenGrab)
	{
		vec3 Normal = texture2D(Texture3Sampler, texCoord.xy).xyz;	
		Normal = (Normal * 2.0) - 1.0;
		Normal.y = -Normal.y;
	
		float distortPower = 0.0;
		
		if (Color.a > 0.1)
			distortPower = 0.03 * (1.0 - Color.a);
        vec2 screenGrabCoord = v_texCoord.xy;
        screenGrabCoord.y = 1.0 - screenGrabCoord.y;
		vec2 NormDistortion = screenGrabCoord + (Normal.xy * distortPower);
		NormDistortion.x = clamp(NormDistortion.x, 0.00, 1.00);
		NormDistortion.y = clamp(NormDistortion.y, 0.00, 1.00);
	
		//get Screen Texture
		vec4 ScreenTex = texture2D(Texture4Sampler, NormDistortion);
		
		//layer fire on top
		Color.rgb = lerp(ScreenTex.rgb, Color.rgb, Color.a);
		Color.a = 1.0;
	}

    gl_FragColor = clamp(Color, 0.0, 1.0);
}